@@ -33,8 +33,6 @@
 #include "mate-settings-plugin.h"
 #include "mate-settings-profile.h"
 
-#define MATE_SETTINGS_PLUGIN_INFO_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), MATE_TYPE_SETTINGS_PLUGIN_INFO, MateSettingsPluginInfoPrivate))
-
 #define PLUGIN_GROUP "MATE Settings Plugin"
 
 #define PLUGIN_PRIORITY_MAX 1
@@ -70,7 +68,6 @@ struct MateSettingsPluginInfoPrivate
         int                      priority;
 };
 
-
 enum {
         ACTIVATED,
         DEACTIVATED,
@@ -149,7 +146,7 @@ mate_settings_plugin_info_class_init (MateSettingsPluginInfoClass *klass)
 static void
 mate_settings_plugin_info_init (MateSettingsPluginInfo *info)
 {
-        info->priv = MATE_SETTINGS_PLUGIN_INFO_GET_PRIVATE (info);
+        info->priv = mate_settings_plugin_info_get_instance_private (info);
 }
 
 static void
@@ -319,7 +316,6 @@ mate_settings_plugin_info_deactivate (MateSettingsPluginInfo *info)
         return TRUE;
 }
 
-
 static gboolean
 load_plugin_module (MateSettingsPluginInfo *info)
 {
@@ -481,7 +477,6 @@ mate_settings_plugin_info_get_copyright (MateSettingsPluginInfo *info)
         return info->priv->copyright;
 }
 
-
 const char *
 mate_settings_plugin_info_get_location (MateSettingsPluginInfo *info)
 {
